@@ -33,22 +33,22 @@ public class Version {
         "java";
 
     private static final String java_version =
-        "1.7.0_99";
+        "1.7.0_131";
 
     private static final String java_runtime_name =
         "OpenJDK Runtime Environment";
 
     private static final String java_runtime_version =
-        "1.7.0_99-b00";
+        "1.7.0_131-b00";
 
     private static final String jdk_derivative_name =
-        "IcedTea 2.6.5";
+        "IcedTea 2.6.9";
 
     private static final String distro_name =
         "linux-gnueabihf";
 
     private static final String distro_package_version =
-        "Arch Linux ARM build 7.u99_2.6.5-1-armv7h";
+        "Arch Linux ARM build 7.u131_2.6.9-1-armv7h";
 
     private static final String jdk_revid =
         "";
@@ -313,15 +313,24 @@ public class Version {
                 jvm_minor_version = Character.digit(cs.charAt(2), 10);
                 jvm_micro_version = Character.digit(cs.charAt(4), 10);
                 cs = cs.subSequence(5, cs.length());
-                if (cs.charAt(0) == '_' && cs.length() >= 3 &&
-                    Character.isDigit(cs.charAt(1)) &&
-                    Character.isDigit(cs.charAt(2))) {
-                    int nextChar = 3;
+                if (cs.charAt(0) == '_' && cs.length() >= 3) {
+                    int nextChar = 0;
+                    if (Character.isDigit(cs.charAt(1)) &&
+                        Character.isDigit(cs.charAt(2)) &&
+                        Character.isDigit(cs.charAt(3)))
+                    {
+                        nextChar = 4;
+                    } else if (Character.isDigit(cs.charAt(1)) &&
+                        Character.isDigit(cs.charAt(2)))
+                    {
+                        nextChar = 3;
+                    }
+
                     try {
-                        String uu = cs.subSequence(1, 3).toString();
+                        String uu = cs.subSequence(1, nextChar).toString();
                         jvm_update_version = Integer.valueOf(uu).intValue();
-                        if (cs.length() >= 4) {
-                            char c = cs.charAt(3);
+                        if (cs.length() >= nextChar + 1) {
+                            char c = cs.charAt(nextChar);
                             if (c >= 'a' && c <= 'z') {
                                 jvm_special_version = Character.toString(c);
                                 nextChar++;
